# -*- coding: utf-8 -*-

from formats_common import *

enable = False
lname = "gettext_mo"
format = "GettextMo"
description = "Gettext Binary (mo)"
extensions = (".mo",)
extensionCreate = ".mo"
singleFile = True
kind = "binary"
wiki = "https://en.wikipedia.org/wiki/Gettext"
website = (
	"https://www.gnu.org/software/gettext",
	"gettext - GNU Project",
)
optionsProp = {}
